@@ -22,7 +22,7 @@
 #define _GNOME_WP_INFO_H_
 
 #include <glib.h>
-#include <libgnomeui/gnome-desktop-thumbnail.h>
+#include <libgnome-desktop/gnome-desktop-thumbnail.h>
 
 typedef struct _GnomeWPInfo GnomeWPInfo;
 
@@ -38,6 +38,7 @@ struct _GnomeWPInfo {
 };
 
 GnomeWPInfo * gnome_wp_info_new (const gchar * uri,
+				 GFileInfo * file_info,
 				 GnomeDesktopThumbnailFactory * thumbs);
 void gnome_wp_info_free (GnomeWPInfo * info);
 
