@@ -108,13 +108,12 @@ static void set_bg_properties (GnomeWPItem *item)
 
 void gnome_wp_item_ensure_gnome_bg (GnomeWPItem *item)
 {
-  if (!item->bg) {
+  if (!item->bg)
     item->bg = gnome_bg_new ();
 
-    g_object_set_data (G_OBJECT (item->bg), "gnome-wp-item", item);
+  g_object_set_data (G_OBJECT (item->bg), "gnome-wp-item", item);
 
-    set_bg_properties (item);
-  }
+  set_bg_properties (item);
 }
 
 void gnome_wp_item_update (GnomeWPItem *item) {
@@ -125,7 +124,7 @@ void gnome_wp_item_update (GnomeWPItem *item) {
   settings = g_settings_new (WP_PATH_ID);
 
   item->options = g_settings_get_enum (settings, WP_OPTIONS_KEY);
-  item->shade_type = g_settings_get_enum (settings,WP_SHADING_KEY);
+  item->shade_type = g_settings_get_enum (settings, WP_SHADING_KEY);
 
   s = g_settings_get_string (settings, WP_PCOLOR_KEY);
   if (s != NULL) {
@@ -155,11 +154,12 @@ void gnome_wp_item_update (GnomeWPItem *item) {
 
 GnomeWPItem * gnome_wp_item_new (const gchar * filename,
 				 GHashTable * wallpapers,
+				 GFileInfo * file_info,
 				 GnomeDesktopThumbnailFactory * thumbnails) {
   GnomeWPItem *item = g_new0 (GnomeWPItem, 1);
 
   item->filename = g_strdup (filename);
-  item->fileinfo = gnome_wp_info_new (filename, thumbnails);
+  item->fileinfo = gnome_wp_info_new (filename, file_info, thumbnails);
 
   if (item->fileinfo != NULL && item->fileinfo->mime_type != NULL &&
       (g_str_has_prefix (item->fileinfo->mime_type, "image/") ||
@@ -173,7 +173,7 @@ GnomeWPItem * gnome_wp_item_new (const gchar * filename,
 
     gnome_wp_item_update (item);
     gnome_wp_item_ensure_gnome_bg (item);
-    gnome_wp_item_update_description (item);
+    gnome_wp_item_update_size (item, NULL);
 
     if (wallpapers)
       g_hash_table_insert (wallpapers, item->filename, item);
@@ -192,7 +192,7 @@ void gnome_wp_item_free (GnomeWPItem * item) {
 
   g_free (item->name);
   g_free (item->filename);
-  g_free (item->description);
+  g_free (item->size);
 
   if (item->pcolor != NULL)
     gdk_color_free (item->pcolor);
@@ -211,131 +211,83 @@ void gnome_wp_item_free (GnomeWPItem * item) {
   g_free (item);
 }
 
-static GdkPixbuf *
-add_slideshow_frame (GdkPixbuf *pixbuf)
+static GEmblem *
+get_slideshow_icon (void)
 {
-  GdkPixbuf *sheet, *sheet2;
-  GdkPixbuf *tmp;
-  gint w, h;
+	GIcon *themed;
+	GEmblem *emblem;
 
-  w = gdk_pixbuf_get_width (pixbuf);
-  h = gdk_pixbuf_get_height (pixbuf);
-
-  sheet = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
-  gdk_pixbuf_fill (sheet, 0x00000000);
-  sheet2 = gdk_pixbuf_new_subpixbuf (sheet, 1, 1, w - 2, h - 2);
-  gdk_pixbuf_fill (sheet2, 0xffffffff);
-  g_object_unref (sheet2);
-
-  tmp = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, w + 6, h + 6);
-
-  gdk_pixbuf_fill (tmp, 0x00000000);
-  gdk_pixbuf_composite (sheet, tmp, 6, 6, w, h, 6.0, 6.0, 1.0, 1.0, GDK_INTERP_NEAREST, 255);
-  gdk_pixbuf_composite (sheet, tmp, 3, 3, w, h, 3.0, 3.0, 1.0, 1.0, GDK_INTERP_NEAREST, 255);
-  gdk_pixbuf_composite (pixbuf, tmp, 0, 0, w, h, 0.0, 0.0, 1.0, 1.0, GDK_INTERP_NEAREST, 255);
-
-  g_object_unref (sheet);
-
-  return tmp;
+	themed = g_themed_icon_new ("slideshow-emblem");
+	emblem = g_emblem_new_with_origin (themed, G_EMBLEM_ORIGIN_DEVICE);
+	g_object_unref (themed);
+	return emblem;
 }
 
-GdkPixbuf * gnome_wp_item_get_frame_thumbnail (GnomeWPItem * item,
+GIcon * gnome_wp_item_get_frame_thumbnail (GnomeWPItem * item,
 					       GnomeDesktopThumbnailFactory * thumbs,
                                                int width,
                                                int height,
                                                gint frame) {
   GdkPixbuf *pixbuf = NULL;
+  GIcon *icon = NULL;
 
   set_bg_properties (item);
 
-  if (frame != -1)
+  if (frame >= 0)
     pixbuf = gnome_bg_create_frame_thumbnail (item->bg, thumbs, gdk_screen_get_default (), width, height, frame);
   else
     pixbuf = gnome_bg_create_thumbnail (item->bg, thumbs, gdk_screen_get_default(), width, height);
 
-  if (pixbuf && gnome_bg_changes_with_time (item->bg))
+  if (pixbuf && frame != -2 && gnome_bg_changes_with_time (item->bg))
     {
-      GdkPixbuf *tmp;
+      GEmblem *emblem;
 
-      tmp = add_slideshow_frame (pixbuf);
+      emblem = get_slideshow_icon ();
+      icon = g_emblemed_icon_new (G_ICON (pixbuf), emblem);
+      g_object_unref (emblem);
       g_object_unref (pixbuf);
-      pixbuf = tmp;
+    }
+  else
+    {
+      icon = G_ICON (pixbuf);
     }
 
   gnome_bg_get_image_size (item->bg, thumbs, width, height, &item->width, &item->height);
 
-  return pixbuf;
+  return icon;
 }
 
 
-GdkPixbuf * gnome_wp_item_get_thumbnail (GnomeWPItem * item,
+GIcon * gnome_wp_item_get_thumbnail (GnomeWPItem * item,
 					 GnomeDesktopThumbnailFactory * thumbs,
                                          gint width,
                                          gint height) {
   return gnome_wp_item_get_frame_thumbnail (item, thumbs, width, height, -1);
 }
 
-void gnome_wp_item_update_description (GnomeWPItem * item) {
-  g_free (item->description);
+void gnome_wp_item_update_size (GnomeWPItem * item,
+				GnomeDesktopThumbnailFactory * thumbs) {
+  g_free (item->size);
+  item->size = NULL;
 
   if (!strcmp (item->filename, "(none)")) {
-    item->description = g_strdup (item->name);
+    item->size = g_strdup ("");
   } else {
-    gchar *description;
-    gchar *size;
-    gchar *dirname = g_path_get_dirname (item->filename);
-
-    description = NULL;
-    size = NULL;
-
-    if (strcmp (item->fileinfo->mime_type, "application/xml") == 0)
-      {
-        if (gnome_bg_changes_with_time (item->bg))
-          description = g_strdup (_("Slide Show"));
-        else if (item->width > 0 && item->height > 0)
-          description = g_strdup (_("Image"));
+    if (gnome_bg_has_multiple_sizes (item->bg) || gnome_bg_changes_with_time (item->bg))
+      item->size = g_strdup (_("multiple sizes"));
+    else {
+      if (thumbs != NULL && (item->width <= 0 || item->height <= 0)) {
+        gnome_bg_get_image_size (item->bg, thumbs, 1, 1, &item->width, &item->height);
+      }
+      if (item->width > 0 && item->height > 0) {
+        /* translators: 100 × 100px
+         * Note that this is not an "x", but U+00D7 MULTIPLICATION SIGN */
+        item->size = g_strdup_printf (_("%d \303\227 %d"),
+				      item->width,
+				      item->height);
+      } else {
+        item->size = g_strdup ("");
       }
-    else
-      description = g_content_type_get_description (item->fileinfo->mime_type);
-
-    if (gnome_bg_has_multiple_sizes (item->bg))
-      size = g_strdup (_("multiple sizes"));
-    else if (item->width > 0 && item->height > 0) {
-      /* translators: x pixel(s) by y pixel(s) */
-      size = g_strdup_printf (_("%d %s by %d %s"),
-                              item->width,
-                              ngettext ("pixel", "pixels", item->width),
-                              item->height,
-                              ngettext ("pixel", "pixels", item->height));
     }
-
-    if (description && size) {
-      /* translators: <b>wallpaper name</b>
-       * mime type, size
-       * Folder: /path/to/file
-       */
-      item->description = g_markup_printf_escaped (_("<b>%s</b>\n"
-                                                     "%s, %s\n"
-                                                     "Folder: %s"),
-                                                   item->name,
-                                                   description,
-                                                   size,
-                                                   dirname);
-    } else {
-      /* translators: <b>wallpaper name</b>
-       * Image missing
-       * Folder: /path/to/file
-       */
-      item->description = g_markup_printf_escaped (_("<b>%s</b>\n"
-                                                     "%s\n"
-                                                     "Folder: %s"),
-                                                   item->name,
-                                                   _("Image missing"),
-                                                   dirname);
-    }
-
-    g_free (size);
-    g_free (dirname);
-    g_free (description);
   }
 }
