@@ -21,6 +21,7 @@
 
 #include "cc-datetime-panel.h"
 
+#include <sys/time.h>
 #include "cc-timezone-map.h"
 #include "set-timezone.h"
 #include <gsettings-desktop-schemas/gdesktop-enums.h>
@@ -69,9 +70,12 @@ struct _CcDateTimePanelPrivate
 
   GSettings *settings;
   GDesktopClockFormat clock_format;
+
+  guint update_id;
 };
 
 static void update_time (CcDateTimePanel *self);
+static void queue_clock_update (CcDateTimePanel *self);
 
 static void
 cc_date_time_panel_get_property (GObject    *object,
@@ -104,6 +108,12 @@ cc_date_time_panel_dispose (GObject *object)
 {
   CcDateTimePanelPrivate *priv = CC_DATE_TIME_PANEL (object)->priv;
 
+  if (priv->update_id != 0)
+    {
+      g_source_remove (priv->update_id);
+      priv->update_id = 0;
+    }
+
   if (priv->builder)
     {
       g_object_unref (priv->builder);
@@ -148,7 +158,8 @@ static void clock_settings_changed_cb (GSettings       *settings,
                                        CcDateTimePanel *panel);
 
 static void
-change_clock_settings (GtkWidget       *widget,
+change_clock_settings (GObject         *gobject,
+                       GParamSpec      *pspec,
                        CcDateTimePanel *panel)
 {
   CcDateTimePanelPrivate *priv = panel->priv;
@@ -157,10 +168,10 @@ change_clock_settings (GtkWidget       *widget,
   g_signal_handlers_block_by_func (priv->settings, clock_settings_changed_cb,
                                    panel);
 
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (W ("12_radiobutton"))))
-    value = G_DESKTOP_CLOCK_FORMAT_12H;
-  else
+  if (gtk_switch_get_active (GTK_SWITCH (W ("24h_time_switch"))))
     value = G_DESKTOP_CLOCK_FORMAT_24H;
+  else
+    value = G_DESKTOP_CLOCK_FORMAT_12H;
 
   g_settings_set_enum (priv->settings, CLOCK_FORMAT_KEY, value);
   priv->clock_format = value;
@@ -177,28 +188,24 @@ clock_settings_changed_cb (GSettings       *settings,
                            CcDateTimePanel *panel)
 {
   CcDateTimePanelPrivate *priv = panel->priv;
-  GtkWidget *radio12, *radio24;
-  gboolean use_12_hour;
+  GtkWidget *switch24h;
+  gboolean use_24_hour;
   GDesktopClockFormat value;
 
   value = g_settings_get_enum (settings, CLOCK_FORMAT_KEY);
   priv->clock_format = value;
 
-  radio12 = W ("12_radiobutton");
-  radio24 = W ("24_radiobutton");
+  switch24h = W ("24h_time_switch");
 
-  use_12_hour = (value == G_DESKTOP_CLOCK_FORMAT_12H);
+  use_24_hour = (value == G_DESKTOP_CLOCK_FORMAT_24H);
 
-  g_signal_handlers_block_by_func (radio12, change_clock_settings, panel);
-  g_signal_handlers_block_by_func (radio24, change_clock_settings, panel);
+  g_signal_handlers_block_by_func (switch24h, change_clock_settings, panel);
 
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio12), use_12_hour);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio24), !use_12_hour);
+  gtk_switch_set_active (GTK_SWITCH (switch24h), use_24_hour);
 
   update_time (panel);
 
-  g_signal_handlers_unblock_by_func (radio12, change_clock_settings, panel);
-  g_signal_handlers_unblock_by_func (radio24, change_clock_settings, panel);
+  g_signal_handlers_unblock_by_func (switch24h, change_clock_settings, panel);
 }
 
 static void
@@ -212,7 +219,7 @@ update_time (CcDateTimePanel *self)
   if (priv->clock_format == G_DESKTOP_CLOCK_FORMAT_24H)
     {
       /* Update the hours label */
-      label = g_date_time_format (priv->date, "%H"); 
+      label = g_date_time_format (priv->date, "%H");
       gtk_label_set_text (GTK_LABEL (W("hours_label")), label);
       g_free (label);
     }
@@ -248,6 +255,10 @@ set_time_cb (CcDateTimePanel *self,
     {
       g_warning ("Could not set system time: %s", error->message);
     }
+  else
+    {
+      update_time (self);
+    }
 }
 
 static void
@@ -273,14 +284,41 @@ set_using_ntp_cb (CcDateTimePanel *self,
 }
 
 static void
-apply_button_clicked_cb (GtkButton       *button,
-                         CcDateTimePanel *self)
+queue_set_datetime (CcDateTimePanel *self)
+{
+  time_t unixtime;
+
+  /* for now just do it */
+  unixtime = g_date_time_to_unix (self->priv->date);
+  set_system_time_async (unixtime, (GFunc) set_time_cb, self, NULL);
+}
+
+static void
+queue_set_ntp (CcDateTimePanel *self)
+{
+  CcDateTimePanelPrivate *priv = self->priv;
+  gboolean using_ntp;
+  /* for now just do it */
+  using_ntp = gtk_switch_get_active (GTK_SWITCH (W("network_time_switch")));
+  set_using_ntp_async (using_ntp, (GFunc) set_using_ntp_cb, self, NULL);
+}
+
+static void
+queue_set_timezone (CcDateTimePanel *self)
+{
+  /* for now just do it */
+  if (self->priv->current_location)
+    {
+      set_system_timezone_async (self->priv->current_location->zone, (GFunc) set_timezone_cb, self, NULL);
+    }
+}
+
+static void
+change_date (CcDateTimePanel *self)
 {
   CcDateTimePanelPrivate *priv = self->priv;
   guint mon, y, d;
-  time_t unixtime;
   GDateTime *old_date;
-  gboolean using_ntp;
 
   old_date = priv->date;
 
@@ -293,24 +331,61 @@ apply_button_clicked_cb (GtkButton       *button,
                                       g_date_time_get_minute (old_date),
                                       g_date_time_get_second (old_date));
   g_date_time_unref (old_date);
-
-  unixtime = g_date_time_to_unix (priv->date);
-
-  set_system_time_async (unixtime, (GFunc) set_time_cb, self, NULL);
-
-  if (priv->current_location)
-    {
-      set_system_timezone_async (priv->current_location->zone, (GFunc) set_timezone_cb, self, NULL);
-    }
-
-  using_ntp = gtk_switch_get_active (GTK_SWITCH (W("network_time_switch")));
-  set_using_ntp_async (using_ntp, (GFunc) set_using_ntp_cb, self, NULL);
+  queue_set_datetime (self);
 }
 
 static void
 location_changed_cb (CcTimezoneMap   *map,
                      TzLocation      *location,
                      CcDateTimePanel *self)
+{
+  g_debug ("location changed");
+
+  self->priv->current_location = location;
+
+  queue_set_timezone (self);
+}
+
+static void
+region_changed_cb (GtkComboBox     *box,
+                   CcDateTimePanel *self)
+{
+  GtkTreeModelFilter *modelfilter;
+
+  modelfilter = GTK_TREE_MODEL_FILTER (gtk_builder_get_object (self->priv->builder, "city-modelfilter"));
+
+  gtk_tree_model_filter_refilter (modelfilter);
+}
+
+static void
+city_changed_cb (GtkComboBox     *box,
+                 CcDateTimePanel *self)
+{
+  static gboolean inside = FALSE;
+  GtkTreeIter iter;
+  gchar *zone;
+
+  /* prevent re-entry from location changed callback */
+  if (inside)
+    return;
+
+  inside = TRUE;
+
+  if (gtk_combo_box_get_active_iter (box, &iter))
+    {
+      gtk_tree_model_get (gtk_combo_box_get_model (box), &iter,
+                          CITY_COL_ZONE, &zone, -1);
+
+      cc_timezone_map_set_timezone (CC_TIMEZONE_MAP (self->priv->map), zone);
+
+      g_free (zone);
+    }
+
+  inside = FALSE;
+}
+
+static void
+update_timezone (CcDateTimePanel *self)
 {
   CcDateTimePanelPrivate *priv = self->priv;
   GtkWidget *widget;
@@ -320,15 +395,13 @@ location_changed_cb (CcTimezoneMap   *map,
   GtkTreeIter iter;
   GtkTreeModel *model;
 
-  priv->current_location = location;
-
   /* tz.c updates the local timezone, which means the spin buttons can be
    * updated with the current time of the new location */
 
   t = time (NULL);
   ltime = localtime (&t);
 
-  split = g_strsplit (location->zone, "/", 2);
+  split = g_strsplit (priv->current_location->zone, "/", 2);
 
   /* remove underscores */
   g_strdelimit (split[1], "_", ' ');
@@ -386,10 +459,31 @@ get_timezone_cb (CcDateTimePanel *self,
                  const gchar     *timezone,
                  GError          *error)
 {
+  GtkWidget *widget;
+
   if (error)
-    g_warning ("Could not get current timezone: %s", error->message);
+    {
+      g_warning ("Could not get current timezone: %s", error->message);
+    }
   else
-    cc_timezone_map_set_timezone (CC_TIMEZONE_MAP (self->priv->map), timezone);
+    {
+      cc_timezone_map_set_timezone (CC_TIMEZONE_MAP (self->priv->map), timezone);
+      self->priv->current_location = cc_timezone_map_get_location (CC_TIMEZONE_MAP (self->priv->map));
+      update_timezone (self);
+    }
+
+  /* now that the initial state is loaded set connect the signals */
+  widget = (GtkWidget*) gtk_builder_get_object (self->priv->builder,
+                                                "region_combobox");
+  g_signal_connect (widget, "changed", G_CALLBACK (region_changed_cb), self);
+
+  widget = (GtkWidget*) gtk_builder_get_object (self->priv->builder,
+                                                "city_combobox");
+  g_signal_connect (widget, "changed", G_CALLBACK (city_changed_cb), self);
+
+  g_signal_connect (self->priv->map, "location-changed",
+                    G_CALLBACK (location_changed_cb), self);
+
 }
 
 /* load region and city tree models */
@@ -509,40 +603,6 @@ load_regions_model (GtkListStore *regions, GtkListStore *cities)
                                         GTK_SORT_ASCENDING);
 }
 
-static void
-region_changed_cb (GtkComboBox        *box,
-                   GtkTreeModelFilter *modelfilter)
-{
-  gtk_tree_model_filter_refilter (modelfilter);
-}
-
-static void
-city_changed_cb (GtkComboBox     *box,
-                 CcDateTimePanel *self)
-{
-  static gboolean inside = FALSE;
-  GtkTreeIter iter;
-  gchar *zone;
-
-  /* prevent re-entry from location changed callback */
-  if (inside)
-    return;
-
-  inside = TRUE;
-
-  if (gtk_combo_box_get_active_iter (box, &iter))
-    {
-      gtk_tree_model_get (gtk_combo_box_get_model (box), &iter,
-                          CITY_COL_ZONE, &zone, -1);
-
-      cc_timezone_map_set_timezone (CC_TIMEZONE_MAP (self->priv->map), zone);
-
-      g_free (zone);
-    }
-
-  inside = FALSE;
-}
-
 static void
 update_widget_state_for_ntp (CcDateTimePanel *panel,
                              gboolean         using_ntp)
@@ -553,6 +613,13 @@ update_widget_state_for_ntp (CcDateTimePanel *panel,
   gtk_widget_set_sensitive (W("table2"), !using_ntp);
 }
 
+static void
+day_changed (GtkWidget       *widget,
+             CcDateTimePanel *panel)
+{
+  change_date (panel);
+}
+
 static void
 month_year_changed (GtkWidget       *widget,
                     CcDateTimePanel *panel)
@@ -575,6 +642,8 @@ month_year_changed (GtkWidget       *widget,
 
   if (gtk_spin_button_get_value_as_int (day_spin) > num_days)
     gtk_spin_button_set_value (day_spin, num_days);
+
+  change_date (panel);
 }
 
 static void
@@ -615,14 +684,41 @@ change_time (GtkButton       *button,
   g_date_time_unref (old_date);
 
   update_time (panel);
+  queue_set_datetime (panel);
 }
 
 static void
-change_ntp (GObject    *gobject,
-            GParamSpec *pspec,
-            gpointer    user_data)
+change_ntp (GObject         *gobject,
+            GParamSpec      *pspec,
+            CcDateTimePanel *self)
 {
-  update_widget_state_for_ntp (user_data, gtk_switch_get_active (GTK_SWITCH (gobject)));
+  update_widget_state_for_ntp (self, gtk_switch_get_active (GTK_SWITCH (gobject)));
+  queue_set_ntp (self);
+}
+
+static gboolean
+update_time_timer (CcDateTimePanel *self)
+{
+  g_date_time_unref (self->priv->date);
+  self->priv->date = g_date_time_new_now_local ();
+  update_time (self);
+  queue_clock_update (self);
+  return FALSE;
+}
+
+static void
+queue_clock_update (CcDateTimePanel *self)
+{
+  int timeouttime;
+  struct timeval tv;
+
+  gettimeofday (&tv, NULL);
+  timeouttime = (G_USEC_PER_SEC - tv.tv_usec) / 1000 + 1;
+
+  /* timeout of one minute if we don't care about the seconds */
+  timeouttime += 1000 * (59 - tv.tv_sec % 60);
+
+  self->priv->update_id = g_timeout_add (timeouttime, (GSourceFunc)update_time_timer, self);
 }
 
 static void
@@ -685,6 +781,8 @@ cc_date_time_panel_init (CcDateTimePanel *self)
                                                     num_days + 1, 1, 10, 1);
   gtk_spin_button_set_adjustment (GTK_SPIN_BUTTON (W ("day-spinbutton")),
                                   adjustment);
+  g_signal_connect (G_OBJECT (W("day-spinbutton")), "value-changed",
+                    G_CALLBACK (day_changed), self);
 
   adjustment = (GtkAdjustment*) gtk_adjustment_new (g_date_time_get_year (priv->date),
                                                     G_MINDOUBLE, G_MAXDOUBLE, 1,
@@ -696,8 +794,6 @@ cc_date_time_panel_init (CcDateTimePanel *self)
 
   /* set up timezone map */
   priv->map = widget = (GtkWidget *) cc_timezone_map_new ();
-  g_signal_connect (widget, "location-changed",
-                    G_CALLBACK (location_changed_cb), self);
   gtk_widget_show (widget);
 
   gtk_container_add (GTK_CONTAINER (gtk_builder_get_object (priv->builder,
@@ -715,19 +811,11 @@ cc_date_time_panel_init (CcDateTimePanel *self)
   g_signal_connect (priv->settings, "changed::" CLOCK_FORMAT_KEY,
                     G_CALLBACK (clock_settings_changed_cb), self);
 
-  g_signal_connect (W ("12_radiobutton"), "toggled",
-                    G_CALLBACK (change_clock_settings), self);
-  g_signal_connect (W ("24_radiobutton"), "toggled",
+  g_signal_connect (W("24h_time_switch"), "notify::active",
                     G_CALLBACK (change_clock_settings), self);
 
   update_time (self);
 
-  g_signal_connect ((GtkWidget*) gtk_builder_get_object (priv->builder,
-                                                         "button_apply"),
-                    "clicked",
-                    G_CALLBACK (apply_button_clicked_cb),
-                    self);
-
   get_system_timezone_async ((GetTimezoneFunc) get_timezone_cb, self, NULL);
 
   priv->locations = (GtkTreeModel*) gtk_builder_get_object (priv->builder,
@@ -741,9 +829,6 @@ cc_date_time_panel_init (CcDateTimePanel *self)
 
   widget = (GtkWidget*) gtk_builder_get_object (priv->builder,
                                                 "region_combobox");
-  g_signal_connect (widget, "changed", G_CALLBACK (region_changed_cb),
-                    city_modelfilter);
-
   city_modelsort = GTK_TREE_MODEL_SORT (gtk_builder_get_object (priv->builder, "city-modelsort"));
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (city_modelsort), CITY_COL_CITY_TRANSLATED,
                                         GTK_SORT_ASCENDING);
@@ -752,9 +837,8 @@ cc_date_time_panel_init (CcDateTimePanel *self)
                                           (GtkTreeModelFilterVisibleFunc) city_model_filter_func,
                                           widget,
                                           NULL);
-  widget = (GtkWidget*) gtk_builder_get_object (priv->builder,
-                                                "city_combobox");
-  g_signal_connect (widget, "changed", G_CALLBACK (city_changed_cb), self);
+
+  queue_clock_update (self);
 }
 
 void
