@@ -1,5 +1,6 @@
-/*
- * Copyright (C) 2010 Intel, Inc
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*-
+ *
+ * Copyright (C) 2010 Richard Hughes <richard@hughsie.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * Author: Thomas Wood <thomas.wood@intel.com>
- *
  */
 
 
@@ -30,24 +29,24 @@ G_BEGIN_DECLS
 #define CC_TYPE_NETWORK_PANEL cc_network_panel_get_type()
 
 #define CC_NETWORK_PANEL(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
-  CC_TYPE_NETWORK_PANEL, CcNetworkPanel))
+	(G_TYPE_CHECK_INSTANCE_CAST ((obj), \
+	CC_TYPE_NETWORK_PANEL, CcNetworkPanel))
 
 #define CC_NETWORK_PANEL_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST ((klass), \
-  CC_TYPE_NETWORK_PANEL, CcNetworkPanelClass))
+	(G_TYPE_CHECK_CLASS_CAST ((klass), \
+	CC_TYPE_NETWORK_PANEL, CcNetworkPanelClass))
 
 #define CC_IS_NETWORK_PANEL(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
-  CC_TYPE_NETWORK_PANEL))
+	(G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
+	CC_TYPE_NETWORK_PANEL))
 
 #define CC_IS_NETWORK_PANEL_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_TYPE ((klass), \
-  CC_TYPE_NETWORK_PANEL))
+	(G_TYPE_CHECK_CLASS_TYPE ((klass), \
+	CC_TYPE_NETWORK_PANEL))
 
 #define CC_NETWORK_PANEL_GET_CLASS(obj) \
-  (G_TYPE_INSTANCE_GET_CLASS ((obj), \
-  CC_TYPE_NETWORK_PANEL, CcNetworkPanelClass))
+	(G_TYPE_INSTANCE_GET_CLASS ((obj), \
+	CC_TYPE_NETWORK_PANEL, CcNetworkPanelClass))
 
 typedef struct _CcNetworkPanel CcNetworkPanel;
 typedef struct _CcNetworkPanelClass CcNetworkPanelClass;
@@ -55,23 +54,19 @@ typedef struct _CcNetworkPanelPrivate CcNetworkPanelPrivate;
 
 struct _CcNetworkPanel
 {
-  CcPanel parent;
-
-  CcNetworkPanelPrivate *priv;
+	CcPanel parent;
+	CcNetworkPanelPrivate *priv;
 };
 
 struct _CcNetworkPanelClass
 {
-  CcPanelClass parent_class;
+	CcPanelClass parent_class;
 };
 
 GType cc_network_panel_get_type (void) G_GNUC_CONST;
 
 void  cc_network_panel_register (GIOModule *module);
 
-
-int gnome_network_properties_init (GtkBuilder  *builder);
-
 G_END_DECLS
 
 #endif /* _CC_NETWORK_PANEL_H */
