@@ -17,7 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * Author: Thomas Wood <thomas.wood@intel.com>
+ * Authors: Thomas Wood <thomas.wood@intel.com>
+ *          Rodrigo Moya <rodrigo@gnome.org>
  *
  */
 
@@ -27,12 +28,6 @@
 
 #include "gconf-property-editor.h"
 
-
-#define CONFIG_ROOT "/desktop/gnome/accessibility"
-
-#define KEY_CONFIG_ROOT CONFIG_ROOT "/keyboard"
-#define MOUSE_CONFIG_ROOT CONFIG_ROOT "/mouse"
-
 #define WID(b, w) (GtkWidget *) gtk_builder_get_object (b, w)
 
 
@@ -46,6 +41,9 @@ struct _CcUaPanelPrivate
   GtkBuilder *builder;
   GConfClient *client;
   GSettings *interface_settings;
+  GSettings *kb_settings;
+  GSettings *mouse_settings;
+  GSettings *font_settings;
 
   GSList *notify_list;
 };
@@ -115,6 +113,24 @@ cc_ua_panel_dispose (GObject *object)
       priv->interface_settings = NULL;
     }
 
+  if (priv->kb_settings)
+    {
+      g_object_unref (priv->kb_settings);
+      priv->kb_settings = NULL;
+    }
+
+  if (priv->mouse_settings)
+    {
+      g_object_unref (priv->mouse_settings);
+      priv->mouse_settings = NULL;
+    }
+
+  if (priv->font_settings)
+    {
+      g_object_unref (priv->font_settings);
+      priv->font_settings = NULL;
+    }
+
   G_OBJECT_CLASS (cc_ua_panel_parent_class)->dispose (object);
 }
 
@@ -197,6 +213,26 @@ cc_ua_panel_section_toggled (GtkToggleButton *button,
     }
 }
 
+static void
+settings_on_off_editor_new (CcUaPanelPrivate  *priv,
+                            GSettings         *settings,
+                            const gchar       *key,
+                            GtkWidget         *widget,
+                            gchar            **section)
+{
+  /* set data to enable/disable the section this on/off switch controls */
+  if (section)
+    {
+      g_object_set_data (G_OBJECT (widget), "section-widgets", section);
+      g_signal_connect (widget, "toggled",
+                        G_CALLBACK (cc_ua_panel_section_toggled),
+                        priv->builder);
+    }
+
+  /* set up the boolean editor */
+  g_settings_bind (settings, key, widget, "active", G_SETTINGS_BIND_DEFAULT);
+}
+
 static GConfValue*
 cc_ua_panel_toggle_radios (GConfPropertyEditor *peditor,
                            const GConfValue    *value)
@@ -207,7 +243,6 @@ cc_ua_panel_toggle_radios (GConfPropertyEditor *peditor,
   enabled = gconf_value_get_bool (value);
   radio = (GtkWidget*) gconf_property_editor_get_ui_control (peditor);
 
-
   if (!enabled)
     {
       GSList *list, *l;
@@ -252,8 +287,7 @@ gconf_on_off_peditor_new (CcUaPanelPrivate  *priv,
   peditor = gconf_peditor_new_boolean (NULL, key, widget, NULL);
   g_object_set (peditor, "conv-to-widget-cb", cc_ua_panel_toggle_radios, NULL);
 
-  /* emit the notify on the key, so that the conv-to-widget-cb callback is run
-   */
+  /* emit the notify on the key, so that the conv-to-widget-cb callback is run */
   gconf_client_notify (priv->client, key);
 }
 
@@ -263,8 +297,6 @@ gconf_on_off_peditor_new (CcUaPanelPrivate  *priv,
 #define CONTRAST_MODEL_THEME_COLUMN 3
 #define DPI_MODEL_FACTOR_COLUMN 2
 
-#define DPI_KEY "/desktop/gnome/font_rendering/dpi"
-
 /* The following two functions taken from gsd-a11y-preferences-dialog.c
  *
  * Copyright (C)  2008 William Jon McCann <jmccann@redhat.com>
@@ -280,7 +312,7 @@ gconf_on_off_peditor_new (CcUaPanelPrivate  *priv,
  * DPI_HIGH_REASONABLE_VALUE], then we assume that it is lying and we use
  * DPI_FALLBACK instead.
  *
- * See get_dpi_from_gconf_or_server() below, and also
+ * See get_dpi_from_x_server() below, and also
  * https://bugzilla.novell.com/show_bug.cgi?id=217790
  */
 #define DPI_LOW_REASONABLE_VALUE 50
@@ -335,9 +367,8 @@ get_dpi_from_x_server ()
 }
 
 static void
-dpi_notify_cb (GConfClient *client,
-               guint        cnxn_id,
-               GConfEntry  *entry,
+dpi_notify_cb (GSettings   *settings,
+               const gchar *key,
                CcUaPanel   *panel)
 {
   CcUaPanelPrivate *priv = panel->priv;
@@ -345,13 +376,13 @@ dpi_notify_cb (GConfClient *client,
   GtkTreeModel *model;
   GtkWidget *combo;
   gboolean valid;
-  gdouble gconf_value;
+  gdouble conf_value;
   gdouble x_dpi;
 
-  if (!entry->value)
+  if (!g_str_equal (key, "dpi"))
     return;
 
-  gconf_value = gconf_value_get_float (entry->value);
+  conf_value = g_settings_get_double (settings, key);
 
   combo = WID (priv->builder, "seeing_text_size_combobox");
   model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
@@ -369,7 +400,7 @@ dpi_notify_cb (GConfClient *client,
                           DPI_MODEL_FACTOR_COLUMN, &factor,
                           -1);
 
-      if (gconf_value == (float) (factor * x_dpi))
+      if (conf_value == (float) (factor * x_dpi))
         {
           gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter);
           break;
@@ -400,7 +431,7 @@ dpi_combo_box_changed (GtkComboBox *box,
                       -1);
 
   if (factor == 1.0)
-    gconf_client_unset (priv->client, DPI_KEY, NULL);
+    g_settings_reset (priv->font_settings, "dpi");
   else
     {
       gdouble x_dpi, u_dpi;
@@ -408,7 +439,7 @@ dpi_combo_box_changed (GtkComboBox *box,
       x_dpi = get_dpi_from_x_server ();
       u_dpi = (gdouble) factor * x_dpi;
 
-      gconf_client_set_float (priv->client, DPI_KEY, u_dpi, NULL);
+      g_settings_set_double (priv->font_settings, "dpi", u_dpi);
     }
 }
 
@@ -494,28 +525,18 @@ static void
 cc_ua_panel_init_seeing (CcUaPanel *self)
 {
   CcUaPanelPrivate *priv = self->priv;
-  guint id;
 
-  gconf_client_add_dir (priv->client, "/desktop/gnome/font_rendering",
-                        GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-
-  id = gconf_client_notify_add (priv->client, DPI_KEY,
-                                (GConfClientNotifyFunc) dpi_notify_cb,
-                                self, NULL, NULL);
-  priv->notify_list = g_slist_prepend (priv->notify_list, GINT_TO_POINTER (id));
+  g_signal_connect (priv->font_settings, "changed", G_CALLBACK (dpi_notify_cb), self);
 
   g_signal_connect (WID (priv->builder, "seeing_contrast_combobox"), "changed",
                     G_CALLBACK (contrast_combobox_changed_cb), self);
 
   g_signal_connect (WID (priv->builder, "seeing_text_size_combobox"), "changed",
                     G_CALLBACK (dpi_combo_box_changed), self);
-  gconf_client_notify (priv->client, DPI_KEY);
 
-  gconf_peditor_new_boolean (NULL,
-                             "/desktop/gnome/accessibility/keyboard/togglekeys_enable",
-                             WID (priv->builder,
-                                  "seeing_enable_toggle_keys_checkbutton"),
-                             NULL);
+  g_settings_bind (priv->kb_settings, "togglekeys-enable",
+                   WID (priv->builder, "seeing_enable_toggle_keys_checkbutton"), "active",
+                   G_SETTINGS_BIND_DEFAULT);
 }
 
 
@@ -605,64 +626,51 @@ static void
 cc_ua_panel_init_keyboard (CcUaPanel *self)
 {
   CcUaPanelPrivate *priv = self->priv;
-  GConfChangeSet *changeset = NULL;
   GtkWidget *w;
 
-
   /* enable shortcuts */
   w = WID (priv->builder, "typing_keyboard_toggle_checkbox");
-  gconf_peditor_new_boolean (changeset, KEY_CONFIG_ROOT "/enable", w, NULL);
+  g_settings_bind (priv->kb_settings, "enable", w, "active", G_SETTINGS_BIND_DEFAULT);
 
   /* sticky keys */
   w = WID (priv->builder, "typing_sticky_keys_on_radiobutton");
-  gconf_on_off_peditor_new (priv, KEY_CONFIG_ROOT "/stickykeys_enable",
-                            w, sticky_keys_section);
+  settings_on_off_editor_new (priv, priv->kb_settings, "stickykeys-enable", w, sticky_keys_section);
 
   w = WID (priv->builder, "typing_sticky_keys_disable_two_keys_checkbutton");
-  gconf_peditor_new_boolean (changeset,
-                             KEY_CONFIG_ROOT "/stickykeys_two_key_off", w,
-                             NULL);
+  g_settings_bind (priv->kb_settings, "stickykeys-two-key-off", w, "active", G_SETTINGS_BIND_DEFAULT);
 
   w = WID (priv->builder, "typing_sticky_keys_beep_modifier_checkbutton");
-  gconf_peditor_new_boolean (changeset,
-                             KEY_CONFIG_ROOT "/stickykeys_modifier_beep", w,
-                             NULL);
+  g_settings_bind (priv->kb_settings, "stickykeys-modifier-beep", w, "active", G_SETTINGS_BIND_DEFAULT);
 
   /* slow keys */
   w = WID (priv->builder, "typing_slow_keys_on_radiobutton");
-  gconf_on_off_peditor_new (priv, KEY_CONFIG_ROOT "/slowkeys_enable",
-                            w, slow_keys_section);
+  settings_on_off_editor_new (priv, priv->kb_settings, "slowkeys-enable", w, slow_keys_section);
 
   w = WID (priv->builder, "typing_slowkeys_delay_scale");
-  gconf_peditor_new_numeric_range (changeset, KEY_CONFIG_ROOT "/slowkeys_delay",
-                                   w, NULL);
+  g_settings_bind (priv->kb_settings, "slowkeys-delay",
+                   gtk_range_get_adjustment (GTK_RANGE (w)), "value",
+                   G_SETTINGS_BIND_DEFAULT);
 
   w = WID (priv->builder, "typing_slow_keys_beep_pressed_checkbutton");
-  gconf_peditor_new_boolean (changeset, KEY_CONFIG_ROOT "/slowkeys_beep_press",
-                             w, NULL);
+  g_settings_bind (priv->kb_settings, "slowkeys-beep-press", w, "active", G_SETTINGS_BIND_DEFAULT);
 
   w = WID (priv->builder, "typing_slow_keys_beep_accepted_checkbutton");
-  gconf_peditor_new_boolean (changeset, KEY_CONFIG_ROOT "/slowkeys_beep_accept",
-                             w, NULL);
+  g_settings_bind (priv->kb_settings, "slowkeys-beep-accept", w, "active", G_SETTINGS_BIND_DEFAULT);
 
   w = WID (priv->builder, "typing_slow_keys_beep_rejected_checkbutton");
-  gconf_peditor_new_boolean (changeset, KEY_CONFIG_ROOT "/slowkeys_beep_reject",
-                             w, NULL);
+  g_settings_bind (priv->kb_settings, "slowkeys-beep-reject", w, "active", G_SETTINGS_BIND_DEFAULT);
 
   /* bounce keys */
   w = WID (priv->builder, "typing_bounce_keys_on_radiobutton");
-  gconf_on_off_peditor_new (priv, KEY_CONFIG_ROOT "/bouncekeys_enable", w,
-                            bounce_keys_section);
+  settings_on_off_editor_new (priv, priv->kb_settings, "bouncekeys-enable", w, bounce_keys_section);
 
   w = WID (priv->builder, "typing_bouncekeys_delay_scale");
-  gconf_peditor_new_numeric_range (changeset,
-                                   KEY_CONFIG_ROOT "/bouncekeys_delay", w,
-                                   NULL);
+  g_settings_bind (priv->kb_settings, "bouncekeys-delay",
+                   gtk_range_get_adjustment (GTK_RANGE (w)), "value",
+                   G_SETTINGS_BIND_DEFAULT);
 
   w = WID (priv->builder, "typing_bounce_keys_beep_rejected_checkbutton");
-  gconf_peditor_new_boolean (changeset,
-                             KEY_CONFIG_ROOT "/bouncekeys_beep_reject", w,
-                             NULL);
+  g_settings_bind (priv->kb_settings, "bouncekeys-beep-reject", w, "active", G_SETTINGS_BIND_DEFAULT);
 
   g_signal_connect (WID (priv->builder, "typing_keyboard_preferences_button"),
                     "clicked",
@@ -684,38 +692,34 @@ static void
 cc_ua_panel_init_mouse (CcUaPanel *self)
 {
   CcUaPanelPrivate *priv = self->priv;
-  GConfChangeSet *changeset = NULL;
   GtkWidget *w;
 
   /* mouse keys */
   w = WID (priv->builder, "pointing_mouse_keys_on_radiobutton");
-  gconf_on_off_peditor_new (priv, KEY_CONFIG_ROOT "/mousekeys_enable", w, NULL);
+  settings_on_off_editor_new (priv, priv->kb_settings, "mousekeys-enable", w, NULL);
 
   /* simulated secondary click */
   w = WID (priv->builder, "pointing_second_click_on_radiobutton");
-  gconf_on_off_peditor_new (priv, MOUSE_CONFIG_ROOT "/delay_enable", w,
-                            secondary_click_section);
+  settings_on_off_editor_new (priv, priv->mouse_settings, "secondary-click-enabled", w, secondary_click_section);
 
   w = WID (priv->builder, "pointing_secondary_click_delay_scale");
-  gconf_peditor_new_numeric_range (changeset,
-                                   MOUSE_CONFIG_ROOT "/delay_time", w,
-                                   NULL);
-
+  g_settings_bind (priv->mouse_settings, "secondary-click-time",
+                   gtk_range_get_adjustment (GTK_RANGE (w)), "value",
+                   G_SETTINGS_BIND_DEFAULT);
 
   /* dwell click */
   w = WID (priv->builder, "pointing_hover_click_on_radiobutton");
-  gconf_on_off_peditor_new (priv,MOUSE_CONFIG_ROOT "/dwell_enable", w,
-                            dwell_click_section);
+  settings_on_off_editor_new (priv, priv->mouse_settings, "dwell-click-enabled", w, dwell_click_section);
 
   w = WID (priv->builder, "pointing_dwell_delay_scale");
-  gconf_peditor_new_numeric_range (changeset,
-                                   MOUSE_CONFIG_ROOT "/dwell_time", w,
-                                   NULL);
+  g_settings_bind (priv->mouse_settings, "dwell-time",
+                   gtk_range_get_adjustment (GTK_RANGE (w)), "value",
+                   G_SETTINGS_BIND_DEFAULT);
 
   w = WID (priv->builder, "pointing_dwell_threshold_scale");
-  gconf_peditor_new_numeric_range (changeset,
-                                   MOUSE_CONFIG_ROOT "/threshold", w,
-                                   NULL);
+  g_settings_bind (priv->mouse_settings, "dwell-threshold",
+                   gtk_range_get_adjustment (GTK_RANGE (w)), "value",
+                   G_SETTINGS_BIND_DEFAULT);
 
   /* mouse preferences button */
   g_signal_connect (WID (priv->builder, "pointing_mouse_preferences_button"),
@@ -761,13 +765,14 @@ cc_ua_panel_init (CcUaPanel *self)
 
   priv->client = gconf_client_get_default ();
 
-  gconf_client_add_dir (priv->client, CONFIG_ROOT,
-                        GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-
   priv->interface_settings = g_settings_new ("org.gnome.desktop.interface");
   g_signal_connect (priv->interface_settings, "changed",
                     G_CALLBACK (interface_settings_changed_cb), self);
 
+  priv->kb_settings = g_settings_new ("org.gnome.desktop.a11y.keyboard");
+  priv->mouse_settings = g_settings_new ("org.gnome.desktop.a11y.mouse");
+  priv->font_settings = g_settings_new ("org.gnome.settings-daemon.plugins.xsettings");
+
   cc_ua_panel_init_keyboard (self);
   cc_ua_panel_init_mouse (self);
   cc_ua_panel_init_hearing (self);
