@@ -89,12 +89,8 @@ activate_panel (GnomeControlCenter *shell,
                 const gchar        *icon_name)
 {
   GnomeControlCenterPrivate *priv = shell->priv;
-  GAppInfo *appinfo;
-  GError *err = NULL;
-  GdkAppLaunchContext *ctx;
   GType panel_type = G_TYPE_INVALID;
   GList *panels, *l;
-  GKeyFile *key_file;
 
   /* check if there is an plugin that implements this panel */
   panels = g_io_extension_point_get_extensions (priv->extension_point);
@@ -164,44 +160,6 @@ activate_panel (GnomeControlCenter *shell,
           return;
         }
     }
-
-
-  /* if a plugin was not found, then start app directly */
-
-  if (err)
-    {
-      g_warning ("Error starting \"%s\": %s", desktop_file, err->message);
-
-      g_error_free (err);
-      err = NULL;
-
-      return;
-    }
-
-  key_file = g_key_file_new ();
-  g_key_file_load_from_file (key_file, desktop_file, 0, &err);
-
-  appinfo = (GAppInfo*) g_desktop_app_info_new_from_keyfile (key_file);
-
-  g_key_file_free (key_file);
-  key_file = NULL;
-
-
-  ctx = gdk_app_launch_context_new ();
-  gdk_app_launch_context_set_screen (ctx, gdk_screen_get_default ());
-  gdk_app_launch_context_set_timestamp (ctx, priv->last_time);
-
-  g_app_info_launch (appinfo, NULL, G_APP_LAUNCH_CONTEXT (ctx), &err);
-
-  g_object_unref (appinfo);
-  g_object_unref (ctx);
-
-  if (err)
-    {
-      g_warning ("Error starting \"%s\": %s", desktop_file, err->message);
-      g_error_free (err);
-      err = NULL;
-    }
 }
 
 static void
@@ -222,6 +180,12 @@ shell_show_overview_page (GnomeControlCenterPrivate *priv)
                             priv->default_window_icon);
 }
 
+void
+gnome_control_center_set_overview_page (GnomeControlCenter *center)
+{
+  shell_show_overview_page (center->priv);
+}
+
 static void
 item_activated_cb (CcShellCategoryView *view,
                    gchar               *name,
@@ -400,14 +364,18 @@ model_filter_func (GtkTreeModel              *model,
   gchar *name, *target;
   gchar *needle, *haystack;
   gboolean result;
+  gchar **keywords;
 
   gtk_tree_model_get (model, iter, COL_NAME, &name,
-                      COL_SEARCH_TARGET, &target, -1);
+                      COL_SEARCH_TARGET, &target,
+                      COL_KEYWORDS, &keywords,
+                      -1);
 
   if (!priv->filter_string || !name || !target)
     {
       g_free (name);
       g_free (target);
+      g_strfreev (keywords);
       return FALSE;
     }
 
@@ -416,10 +384,24 @@ model_filter_func (GtkTreeModel              *model,
 
   result = (strstr (haystack, needle) != NULL);
 
+  if (!result && keywords)
+    {
+      gint i;
+      gchar *keyword;
+
+      for (i = 0; !result && keywords[i]; i++)
+        {
+          keyword = g_utf8_casefold (keywords[i], -1);
+          result = strstr (keyword, needle) == keyword;
+          g_free (keyword);
+        }
+    }
+
   g_free (name);
   g_free (target);
   g_free (haystack);
   g_free (needle);
+  g_strfreev (keywords);
 
   return result;
 }
@@ -889,6 +871,16 @@ gnome_control_center_class_init (GnomeControlCenterClass *klass)
   shell_class->get_toplevel = _shell_get_toplevel;
 }
 
+static gboolean
+queue_resize (gpointer data)
+{
+  GtkWidget *widget = data;
+
+  gtk_widget_queue_resize (widget);
+
+  return FALSE;
+}
+
 static void
 on_window_size_allocate (GtkWidget          *widget,
                          GtkAllocation      *allocation,
@@ -913,7 +905,7 @@ on_window_size_allocate (GtkWidget          *widget,
           screen = gtk_widget_get_screen (widget);
           monitor = gdk_screen_get_monitor_at_window (screen, window);
           gdk_screen_get_monitor_geometry (screen, monitor, &rect);
-          height = MIN (height + 10, rect.height);
+          height = MIN (height + 10, rect.height - 120);
         }
     }
   else
@@ -921,12 +913,41 @@ on_window_size_allocate (GtkWidget          *widget,
       height = 50;
     }
 
-  gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (self->priv->scrolled_window), height);
+  if (gtk_scrolled_window_get_min_content_height (GTK_SCROLLED_WINDOW (self->priv->scrolled_window)) != height)
+    {
+      g_debug ("Setting min content height: %d", height);
+      gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (self->priv->scrolled_window), height);
+      /*
+       * Queueing a resize out of size-allocate is ignored,
+       * so we have to defer to an idle.
+       */
+      g_idle_add (queue_resize, self->priv->scrolled_window);
+    }
+}
+
+static void
+viewport_style_set_cb (GtkWidget *widget,
+                       GtkStyle  *old_style,
+                       gpointer   user_data)
+{
+  GtkStyle *style;
+
+  /* use "base" colours inside the viewport */
+
+  g_signal_handlers_block_by_func (widget, viewport_style_set_cb, NULL);
+
+  style = gtk_widget_get_style (widget);
+
+  gtk_widget_modify_bg (widget, GTK_STATE_NORMAL,
+                        &style->base[GTK_STATE_NORMAL]);
+
+  g_signal_handlers_unblock_by_func (widget, viewport_style_set_cb, NULL);
 }
 
 static void
 gnome_control_center_init (GnomeControlCenter *self)
 {
+  GtkWidget *widget;
   GError *err = NULL;
   GnomeControlCenterPrivate *priv;
 
@@ -949,6 +970,11 @@ gnome_control_center_init (GnomeControlCenter *self)
 
   priv->notebook = W (priv->builder, "notebook");
   priv->scrolled_window = W (priv->builder, "scrolledwindow1");
+
+  widget = W (priv->builder, "viewport");
+  g_signal_connect (widget, "style-set", G_CALLBACK (viewport_style_set_cb),
+                    NULL);
+
   gtk_widget_set_size_request (priv->scrolled_window, FIXED_WIDTH, -1);
   priv->main_vbox = W (priv->builder, "main-vbox");
   g_signal_connect (priv->scrolled_window, "size-allocate", G_CALLBACK (on_window_size_allocate), self);
@@ -989,5 +1015,5 @@ gnome_control_center_show (GnomeControlCenter *center,
 			   GtkApplication     *app)
 {
   gtk_window_set_application (GTK_WINDOW (center->priv->window), app);
-  gtk_widget_show_all (center->priv->window);
+  gtk_widget_show_all (gtk_bin_get_child (GTK_BIN (center->priv->window)));
 }
